@@ -7,6 +7,125 @@
 #
 
 mclustBootstrapLRT <- function(data, modelName = NULL, 
+                               nboot = 999, level = 0.05, maxG = NULL,
+                               verbose = interactive(), ...)
+{ 
+  if (is.null(modelName)) 
+      stop("A 'modelName' must be provided. Please see help(mclustModelNames) which describes the available models.")
+  modelName <- modelName[1]
+  checkModelName(modelName)
+  if (grepl("X", modelName)) 
+      stop("Specified 'modelName' is only valid for one-component mixture.")
+  if (is.null(maxG)) 
+  { 
+    G <- seq.int(1, 9)
+  } else 
+  {
+    maxG <- as.numeric(maxG)
+    G <- seq.int(1, maxG + 1)
+  }
+  
+  BIC <- mclustBIC(data, G = G, modelNames = modelName, 
+                   warn = FALSE, verbose = FALSE, ...)
+  if (!(modelName %in% attr(BIC, "modelNames"))) 
+      stop("'modelName' not compatibile with data. Please see help(mclustModelNames) which describes the available models.")
+  bic <- BIC[, attr(BIC, "modelNames") == modelName]
+  G <- G[!is.na(BIC)]
+  if (length(G) == 0) 
+      stop(paste("no model", modelName, "can be fitted."))
+  if (all(G == 1)) 
+	{  
+	   warning("only 1-component model could be fitted. No LRT is performed!")
+     return()
+  }
+  if (sum(is.na(bic)) > 0) 
+      warning("some model(s) could not be fitted!")
+  if (verbose)
+	{  
+     flush.console()
+     cat("bootstrapping LRTS ...\n")
+     pbar <- txtProgressBar(min = 0, max = (max(G) - 1) * nboot, style = 3)
+     on.exit(close(pbar))
+  }
+  obsLRTS <- p.value <- vector("numeric", length = max(G) - 1)
+  bootLRTS <- matrix(as.double(NA), nrow = nboot, ncol = max(G) - 1)
+  g <- 0
+  continue <- TRUE
+  while (g < (max(G) - 1) & continue) 
+  {  
+     g <- g + 1
+     # fit model under H0
+     Mod0 <- summary(BIC, data, G = g, modelNames = modelName)
+     # fit model under H1
+     Mod1 <- summary(BIC, data, G = g + 1, modelNames = modelName)
+     # observed LRTS
+     obsLRTS[g] <- 2 * (Mod1$loglik - Mod0$loglik)
+     # bootstrap
+     b <- 0
+     while (b < nboot) 
+     { 
+       b <- b + 1
+       # generate 'parametric' bootstrap sample under H0
+       bootSample <- sim(Mod0$modelName, Mod0$parameters, n = Mod0$n)
+       # compute log-likelihood for model under H0
+       if (Mod0$modelName == "X") 
+       { 
+          loglik0 <- mvnX(data = bootSample[, -1], 
+                          parameters = Mod0$parameters, 
+                          warn = FALSE, ...)$loglik
+       } else 
+       if (Mod0$modelName == "XXX") 
+       {  
+          loglik0 <- mvnXXX(data = bootSample[, -1], 
+                            parameters = Mod0$parameters, 
+                            warn = FALSE, ...)$loglik
+       } else 
+       {
+          param0 <- em(data = bootSample[, -1],
+                       modelName = Mod0$modelName, 
+                       parameters = Mod0$parameters, 
+                       warn = FALSE, ...)$parameters
+          loglik0 <- estep(data = bootSample[, -1],
+                           modelName = Mod0$modelName, 
+                           parameters = param0, 
+                           warn = FALSE, ...)$loglik
+       }
+       # compute log-likelihood for model under H1
+       param1 <- em(data = bootSample[, -1], 
+                    modelName = Mod1$modelName, 
+                    parameters = Mod1$parameters, 
+                    warn = FALSE, ...)$parameters
+       loglik1 <- estep(data = bootSample[, -1],
+                        modelName = Mod1$modelName, 
+                        parameters = param1, 
+                        warn = FALSE, ...)$loglik
+       # compute bootstrap LRT
+       LRTS <- 2 * (loglik1 - loglik0)
+       if(is.na(LRTS)) { b <- b - 1; next() }
+       bootLRTS[b, g] <- LRTS
+       if (verbose) 
+         setTxtProgressBar(pbar, (g - 1) * nboot + b)
+     }
+     p.value[g] <- (1 + sum(bootLRTS[, g] >= obsLRTS[g]))/(nboot + 1)
+     # check if not-significant when no maxG is provided
+     if (is.null(maxG) & p.value[g] > level) 
+     { 
+        continue <- FALSE
+        if (verbose) 
+          setTxtProgressBar(pbar, (max(G) - 1) * nboot)
+     }
+  }
+  
+  out <- list(G = 1:g, 
+              modelName = modelName, 
+              obs = obsLRTS[1:g], 
+              boot = bootLRTS[, 1:g, drop = FALSE], 
+              p.value = p.value[1:g])
+  class(out) <- "mclustBootstrapLRT"
+  return(out)
+}
+
+mclustBootstrapLRT_old <- function(data, modelName = NULL, 
                                nboot = 999, level = 0.05, maxG = NULL, 
                                verbose = interactive(), ...)
 {
@@ -68,7 +187,7 @@ mclustBootstrapLRT <- function(data, modelName = NULL,
     b <- 0
     while(b < nboot)
     { 
-			b <- b + 1
+      b <- b + 1
       # generate 'parametric' bootstrap sample under H0
       bootSample <- sim(Mod0$modelName, Mod0$parameters, n = Mod0$n)
       # fit model under H0
@@ -140,12 +259,14 @@ plot.mclustBootstrapLRT <- function(x, G = 1, hist.col = "grey", hist.border = "
 # Bootstrap inference (standard errors and percentile confidence intervals) 
 #
 
-MclustBootstrap <- function(object, nboot = 999, type = c("bs", "wlbs", "pb", "jk"),
-                            max.nonfit = 10*nboot, verbose = interactive(), ...)
+MclustBootstrap <- function(object, nboot = 999, 
+                            type = c("bs", "wlbs", "pb", "jk"), 
+                            alpha = 1, max.nonfit = 10*nboot, 
+                            verbose = interactive(), ...)
 {
   
-	stopifnot("object must be of class 'Mclust' or 'densityMclust'" =
-	          inherits(object, c("Mclust", "densityMclust")))
+  stopifnot("object must be of class 'Mclust' or 'densityMclust'" =
+            inherits(object, c("Mclust", "densityMclust")))
   
   if(any(type %in% c("nonpara", "wlb")))
     { type <- gsub("nonpara", "bs", type)
@@ -187,7 +308,7 @@ MclustBootstrap <- function(object, nboot = 999, type = c("bs", "wlbs", "pb", "j
     switch(type, 
            "bs" = 
            { 
-						 idx <- sample(seq_len(n), size = n, replace = TRUE)
+             idx <- sample(seq_len(n), size = n, replace = TRUE)
              obj$data <- object$data[idx,]
              obj$z <- object$z[idx,]
              obj$warn <- FALSE
@@ -195,7 +316,8 @@ MclustBootstrap <- function(object, nboot = 999, type = c("bs", "wlbs", "pb", "j
            },
            "wlbs" = 
            { 
-						 w <- rexp(n)
+             # w ~ Dirichlet(alpha, ..., alpha)
+             w <- rgamma(n, shape = as.numeric(alpha), rate = 1)
              # w <- w/mean(w)
              w <- w/max(w)
              mod.boot <- try(do.call("me.weighted", 
@@ -204,7 +326,7 @@ MclustBootstrap <- function(object, nboot = 999, type = c("bs", "wlbs", "pb", "j
            },
            "pb" = 
            { 
-						 obj$data <- do.call("sim", object)[,-1,drop=FALSE]
+             obj$data <- do.call("sim", object)[,-1,drop=FALSE]
              obj$z <- estep(data = obj$data, 
                             modelName = obj$modelName, 
                             parameters = obj$parameters)$z
@@ -213,7 +335,7 @@ MclustBootstrap <- function(object, nboot = 999, type = c("bs", "wlbs", "pb", "j
            },
            "jk" =
            { 
-						 idx <- seq_len(n)[-b]
+             idx <- seq_len(n)[-b]
              obj$data <- object$data[idx,]
              obj$z <- object$z[idx,]
              obj$warn <- FALSE
@@ -279,60 +401,64 @@ summary.MclustBootstrap <- function(object, what = c("se", "ci", "ave"), conf.le
   G <- dims[3]
 
   switch(what,
-    "se" = { out <- list(pro  = apply(object$pro, 2, sd, na.rm=TRUE),
-                         mean = apply(object$mean, c(2,3), sd, na.rm=TRUE),
-                         variance = apply(object$variance, c(2,3,4), sd, na.rm=TRUE))
-             if(object$type == "jk")
-                out <- lapply(out, function(x) 
-                              sqrt(x^2*(nboot-object$nonfit-1)^2/(nboot-object$nonfit)))
-           },
-    "ave" = { out <- list(pro  = apply(object$pro, 2, mean, na.rm=TRUE),
-                          mean = apply(object$mean, c(2,3), mean, na.rm=TRUE),
-                          variance = apply(object$variance, c(2,3,4), mean, na.rm=TRUE))
-           },
-    "ci" = { levels <- c((1-conf.level)/2, (1 + conf.level)/2)
-             if(object$type == "jk")
-             { # bias-corrected ci based on normal-approximation
-               ave <- list(pro  = apply(object$pro, 2, mean, na.rm=TRUE),
-                           mean = apply(object$mean, c(2,3), mean, na.rm=TRUE),
-                           variance = t(sapply(seq.int(d), function(j)
-                                        apply(object$variance[,j,j,], 2, mean, na.rm=TRUE),
-                                        simplify = "array")))
-               se <- list(pro  = apply(object$pro, 2, sd, na.rm=TRUE),
-                          mean = apply(object$mean, c(2,3), sd, na.rm=TRUE),
-                          variance  = t(sapply(seq.int(d), function(j)
-                            apply(object$variance[,j,j,], 2, sd, na.rm=TRUE),
-                            simplify = "array")))
-               se <- lapply(se, function(x) 
-                 sqrt(x^2*(nboot-object$nonfit-1)^2/(nboot-object$nonfit)))
-               zq <- qnorm(max(levels))
-               lnames <- paste0(formatC(levels * 100, format = "fg", width = 1, 
-                                        digits = getOption("digits")), "%")
-               # the code above mimic stats:::format_perc(levels) which can't be used
-               # because format_perc is not exported from stats
-               out <- list(pro = array(as.double(NA), c(2,G),
-                                       dimnames = list(lnames, 1:G)),
-                           mean = array(as.double(NA), dim = c(2,d,G),
-                                        dimnames = list(lnames, 1:d, 1:G)),
-                           variance = array(as.double(NA), dim = c(2,d,G),
-                                            dimnames = list(lnames, 1:d, 1:G)))
-               out$pro[1,] <- ave$pro - zq*se$pro
-               out$pro[2,] <- ave$pro + zq*se$pro
-               out$mean[1,,] <- ave$mean - zq*se$mean
-               out$mean[2,,] <- ave$mean + zq*se$mean
-               out$variance[1,,] <- ave$variance - zq*se$variance
-               out$variance[2,,] <- ave$variance + zq*se$variance
-           } else
-             { # percentile-based ci
-               out <- list(pro = apply(object$pro, 2, quantile, probs = levels, na.rm=TRUE),
-                           mean = apply(object$mean, c(2,3), quantile, probs = levels, na.rm=TRUE))
-               v <- array(as.double(NA), dim = c(2,d,G),
-                          dimnames = dimnames(out$mean))
-               for(j in seq.int(d))
-                 v[,j,] <- apply(object$variance[,j,j,], 2, quantile, probs = levels, na.rm=TRUE)
-               out$variance <- v
-             }
-           }
+    "se" = { 
+      out <- list(pro  = apply(object$pro, 2, sd, na.rm=TRUE),
+                  mean = apply(object$mean, c(2,3), sd, na.rm=TRUE),
+                  variance = apply(object$variance, c(2,3,4), sd, na.rm=TRUE))
+      if(object$type == "jk")
+        out <- lapply(out, function(x) 
+          sqrt(x^2*(nboot-object$nonfit-1)^2/(nboot-object$nonfit)))
+    },
+    "ave" = { 
+      out <- list(pro  = apply(object$pro, 2, mean, na.rm=TRUE),
+                  mean = apply(object$mean, c(2,3), mean, na.rm=TRUE),
+                  variance = apply(object$variance, c(2,3,4), mean, na.rm=TRUE))
+    },
+    "ci" = {
+      levels <- c((1-conf.level)/2, (1 + conf.level)/2)
+      if(object$type == "jk")
+      { # bias-corrected ci based on normal-approximation
+        ave <- list(pro  = apply(object$pro, 2, mean, na.rm=TRUE),
+                    mean = apply(object$mean, c(2,3), mean, na.rm=TRUE),
+                    variance = t(sapply(seq.int(d), function(j)
+                      apply(object$variance[,j,j,], 2, mean, na.rm=TRUE),
+                      simplify = "array")))
+        se <- list(pro  = apply(object$pro, 2, sd, na.rm=TRUE),
+                   mean = apply(object$mean, c(2,3), sd, na.rm=TRUE),
+                   variance  = t(sapply(seq.int(d), function(j)
+                     apply(object$variance[,j,j,], 2, sd, na.rm=TRUE),
+                     simplify = "array")))
+        se <- lapply(se, function(x) 
+          sqrt(x^2*(nboot-object$nonfit-1)^2/(nboot-object$nonfit)))
+        zq <- qnorm(max(levels))
+        lnames <- paste0(formatC(levels * 100, format = "fg", width = 1, 
+                                 digits = getOption("digits")), "%")
+        # the code above mimic stats:::format_perc(levels) which can't be used
+        # because format_perc is not exported from stats
+        out <- list(pro = array(as.double(NA), c(2,G),
+                                dimnames = list(lnames, 1:G)),
+                    mean = array(as.double(NA), dim = c(2,d,G),
+                                 dimnames = list(lnames, 1:d, 1:G)),
+                    variance = array(as.double(NA), dim = c(2,d,G),
+                                     dimnames = list(lnames, 1:d, 1:G)))
+        out$pro[1,] <- ave$pro - zq*se$pro
+        out$pro[2,] <- ave$pro + zq*se$pro
+        out$mean[1,,] <- ave$mean - zq*se$mean
+        out$mean[2,,] <- ave$mean + zq*se$mean
+        out$variance[1,,] <- ave$variance - zq*se$variance
+        out$variance[2,,] <- ave$variance + zq*se$variance
+      } else
+      { # percentile-based ci
+        out <- list(pro = apply(object$pro, 2, quantile, probs = levels, na.rm=TRUE),
+                    mean = apply(object$mean, c(2,3), quantile, probs = levels, na.rm=TRUE))
+        v <- array(as.double(NA), dim = c(2,d,G),
+                   dimnames = dimnames(out$mean))
+        for(j in seq.int(d))
+          v[,j,] <- apply(object$variance[,j,j,,drop=FALSE], 4, 
+                          quantile, probs = levels, na.rm=TRUE)
+        out$variance <- v
+      }
+    }
   )
 
   obj <- append(object[c("modelName", "G", "nboot", "type")],
@@ -399,7 +525,7 @@ print.summary.MclustBootstrap <- function(x, digits = getOption("digits"), ...)
   invisible(x)
 }
 
-plot.MclustBootstrap <- function(x, what = c("pro", "mean", "var"), show.parest = TRUE, show.confint = TRUE, hist.col = "grey", hist.border = "lightgrey", breaks = "Sturges", col = "forestgreen", lwd = 2, lty = 3, xlab = NULL, xlim = NULL, ylim = NULL, ...)
+plot.MclustBootstrap <- function(x, what = c("pro", "mean", "var"), show.parest = TRUE, show.confint = TRUE, hist.col = "grey", hist.border = "lightgrey", breaks = NA, col = "forestgreen", lwd = 2, lty = 3, xlab = NULL, xlim = NULL, ylim = NULL, ...)
 {
   object <- x # Argh.  Really want to use object anyway
   what <- match.arg(what, choices = eval(formals(plot.MclustBootstrap)$what))
@@ -407,9 +533,10 @@ plot.MclustBootstrap <- function(x, what = c("pro", "mean", "var"), show.parest
   d <- dim(object$mean)[2]
   varnames <- rownames(par$mean)
   if(show.confint)
-    { ci <- summary(object, what = "ci", ...)
-      ave <- summary(object, what = "ave", ...) 
-    }
+  { 
+    ci <- summary(object, what = "ci", ...)
+    ave <- summary(object, what = "ave", ...) 
+  }
   
   histBoot <- function(boot, stat, ci, ave, breaks, xlim, ylim, xlab, ...)
   { 
@@ -426,48 +553,54 @@ plot.MclustBootstrap <- function(x, what = c("pro", "mean", "var"), show.parest
   }
 
   switch(what, 
-         "pro" = { xlim <- range(if(is.null(xlim)) pretty(object$pro) else xlim)
-                   for(k in 1:object$G) 
-                       histBoot(object$pro[,k], breaks = breaks, 
-                                stat = par$pro[k], 
-                                ci = ci$pro[,k],
-                                ave = ave$pro[k],
-                                xlim = xlim, ylim = ylim,
-                                xlab = ifelse(is.null(xlab),
-                                              paste("Mix. prop. for comp.",k),
-                                              xlab))
+         "pro" = { 
+           xlim <- range(if(is.null(xlim)) pretty(object$pro) else xlim)
+           for(k in 1:object$G) 
+             histBoot(object$pro[,k], 
+                      breaks = if(is.na(breaks)) nclass.numpy else breaks, 
+                      stat = par$pro[k], 
+                      ci = ci$pro[,k],
+                      ave = ave$pro[k],
+                      xlim = xlim, ylim = ylim,
+                      xlab = ifelse(is.null(xlab),
+                                    paste("Mix. prop. for comp.",k),
+                                    xlab))
          },
-         "mean" = { isNull_xlim <- is.null(xlim)
-                    for(j in 1:d)
-                       { xlim <- range(if(isNull_xlim) pretty(object$mean[,j,]) 
-                                       else xlim)
-                         for(k in 1:object$G)
-                            histBoot(object$mean[,j,k], breaks = breaks, 
-                                     stat = par$mean[j,k], 
-                                     ci = ci$mean[,j,k],
-                                     ave = ave$mean[j,k],
-                                     xlim = xlim, ylim = ylim,
-                                     xlab = ifelse(is.null(xlab),
-                                              paste(varnames[j], "mean for comp.",k),
-                                              xlab))
-                       }
+         "mean" = { 
+           isNull_xlim <- is.null(xlim)
+           for(j in 1:d)
+           { 
+             xlim <- range(if(isNull_xlim) pretty(object$mean[,j,]) else xlim)
+             for(k in 1:object$G)
+               histBoot(object$mean[,j,k], 
+                        breaks = if(is.na(breaks)) nclass.numpy else breaks, 
+                        stat = par$mean[j,k], 
+                        ci = ci$mean[,j,k],
+                        ave = ave$mean[j,k],
+                        xlim = xlim, ylim = ylim,
+                        xlab = ifelse(is.null(xlab),
+                                      paste(varnames[j], "mean for comp.",k),
+                                      xlab))
+           }
          },
-         "var" = { isNull_xlim <- is.null(xlim)
-                   for(j in 1:d)
-                      { xlim <- range(if(isNull_xlim) pretty(object$variance[,j,j,]) 
-                                       else xlim)
-                        for(k in 1:object$G)
-                            histBoot(object$variance[,j,j,k], breaks = breaks, 
-                                     stat = par$variance[j,j,k], 
-                                     ci = ci$variance[,j,k],
-                                     ave = ave$variance[j,k],
-                                     xlim = xlim, ylim = ylim,
-                                     xlab = ifelse(is.null(xlab),
-                                              paste(varnames[j], "var. for comp.",k),
-                                              xlab))
-                       }
+         "var" = { 
+           isNull_xlim <- is.null(xlim)
+           for(j in 1:d)
+           { 
+             xlim <- range(if(isNull_xlim) pretty(object$variance[,j,j,]) else xlim)
+             for(k in 1:object$G)
+               histBoot(object$variance[,j,j,k], 
+                        breaks = if(is.na(breaks)) nclass.numpy else breaks,
+                        stat = par$variance[j,j,k], 
+                        ci = ci$variance[,j,k],
+                        ave = ave$variance[j,j,k],
+                        xlim = xlim, ylim = ylim,
+                        xlab = ifelse(is.null(xlab),
+                                      paste(varnames[j], "var. for comp.",k),
+                                      xlab))
+           }
          }
-        )  
+  )  
   invisible()
 }
 
